#include <string.h>
#include <getopt.h>
#include <stdio.h>

/* Convert statistics files to one pedestal file
   format of statistics file:
   H=HPH_<fec>_<altroch>         M= <mean> RMS=   <rms>
   format of pedestal file:
   histchannel mean min max
   gcc -o /tmp/slask stat2summary.c
*/
int decodeFiles(int argc, char **argv)
{
  int i,j,lines;
  int polarity,gain,shaper,dac,preamp;
  FILE *fin,*fout1,*fout2,*fsum1,*fsum2;
  int ch,imean,values;
  int meanhist[1024],rmshist[1024],irms;
  float maxped = 0.0;
  float minped = 1024.0;
  float maxrms = 0;
  float minrms = 1024;
  char msgbuf[512],txt1[16],txt2[16],prefix[16];
  float mean,rms;
  int ped;
  int entries;
  unsigned int shiftreg;
  int pcasetting;
  char *ptr;

  int fec1,fec2;
  int ch1,ch2;

  float sumped,sumrms,meanped,meanrms;

  fec1 = 21;
  fec2 = 21;
  
  ch1 = fec1 * 128;
  ch2 = fec2 * 128 + 127;

  if (argc >= 2) {
    sprintf(msgbuf,"summary-%d.dat1",fec1);
    fsum1 = fopen(msgbuf,"a");
    sprintf(msgbuf,"summary-%d.dat2",fec1);
    fsum2 = fopen(msgbuf,"a");
    for (i = 1; i < argc; i++) {
      if (fin = fopen(argv[i],"r")) {
	memset(meanhist,0,sizeof(meanhist));
	memset(rmshist,0,sizeof(rmshist));
	entries = 0;
	maxped = 0.0;
	minped = 1024.0;
	maxrms = 0.0;
	minrms = 1024.0;
	sumped = 0.0;
	sumrms = 0.0;
	sprintf(msgbuf,"%s1",argv[i]);
	fout1 = fopen(msgbuf,"w");
	sprintf(msgbuf,"%s2",argv[i]);
	fout2 = fopen(msgbuf,"w");
	fprintf(stderr,"#%s\n",argv[i]);
	fgets(msgbuf,sizeof(msgbuf),fin);
	while(!feof(fin)) {
	  if ((*msgbuf == '#') || (*msgbuf == '*')) {
	    if (ptr = strstr(msgbuf,"*PCA")) {
	      sscanf(ptr,"%s %X",prefix,&pcasetting);
	      shiftreg = pcasetting >> 16;
	      dac = (pcasetting & 0xFFFF);
	      polarity = (shiftreg >> 7) & 1;
	      preamp = shiftreg & 1;
	      gain = ((shiftreg >> 1) & 3);
	      shaper = ((shiftreg >> 3) & 7);
	      //printf("po %d pe %d gain %d shaper %d dac %d\n",polarity,preamp,gain,shaper,dac);
	      //	      return;
	    }
	  }
	  else {
	    sscanf(msgbuf,"%d %d %f %f %d",&ch,&imean,&mean,&rms,&values);
	    if ((values > 10000) && (imean > 0) && (ch >= ch1) && (ch <= ch2)) {
	      meanhist[(int)mean]++;
	      irms = (int)(rms*100.0);
	      if (irms < 1000) rmshist[irms]++;
	      if (mean > maxped) maxped = mean;
	      else if (mean < minped) minped = mean;
	      if (rms > maxrms) maxrms = rms;
	      else if (rms < minrms) minrms = rms;
	      entries++;
	      sumped += mean;
	      sumrms += rms;
	    }
	  }
	  fgets(msgbuf,sizeof(msgbuf),fin);
	}
	fclose(fin);
	for (j = 0; j < 1024; j++) {
	  fprintf(fout1,"%10d %10d\n",j,meanhist[j]);
	  fprintf(fout2,"%6.2f %10d\n",(float)j/100.0,rmshist[j]);
	}
	meanped = sumped/(float)entries;
	meanrms = sumrms/(float)entries;
	fclose(fout1);
	fclose(fout2);
	lines++;
	lines = gain * 100000 + shaper * 10000 + dac * 3;
	lines += polarity * 500000;
	//printf("POL %d GAIN %d shaper %d dac %d lines %d\n",polarity,gain,shaper,dac,lines);
	sprintf(msgbuf,"p%d-gain%d-shaper%d-dac%d.dat",polarity,gain,shaper,dac);
	printf("p%d-gain%d-shaper%d-dac%d",polarity,gain,shaper,dac);
	fprintf(fsum1,"%6d %6.2f %6.2f %6.2f %s\n",lines,meanped,minped,maxped,msgbuf);
	fprintf(fsum2,"%6d %6.2f %6.2f %6.2f %s\n",lines,meanrms,minrms,maxrms,msgbuf);
      }
      else {
	fprintf(stderr,"#%s - *ERROR* not found SKIPPED\n", argv[i]);
      }
    }
    fclose(fsum1);
    fclose(fsum2);
    fout1 = fopen("lines.txt","w");
    fprintf(fout1,"%d",lines);
    fclose(fout1);
  }
  else {
    puts("You must enter at least one filename");
    puts("The output is to terminal");
    printf("Usage: %s infile1 ... infileN > outfile\n",argv[0]); 
  }
}
int main(int argc, char **argv)
{
  decodeFiles(argc,argv);
}
